<div class="peek-nav-section"></div>

<div class="peek-nav-bar-padding">
    <peek-core-device-cfg-connect></peek-core-device-cfg-connect>
</div>

<div class="peek-nav-bar-padding">
    <div
        class="peek-information-section"
        *ngFor="let status of (statusList$ | async)"
    >
        <h4>{{status.pluginName}} - {{status.indexName}}</h4>
        <p>
            Load Complete : <b>{{status.initialFullLoadComplete}}</b><br />
            Load Progress : Local count is
            <b>{{status.loadingQueueCount}}</b> chunks, server count is
            <b>{{status.totalLoadedCount}}</b> chunks<br />
            Last Check Finished : {{status.lastCheckDate}}<br />
        </p>
    </div>
</div>
