<div *ngIf="(statusList$ | async).length !== 0; else emptyTpl">
    <div *ngFor="let status of statusList$ | async">
        <h4>{{ status.pluginName }} - {{ status.indexName }}</h4>
        <p>
            Load Complete :
            {{ status.initialFullLoadComplete }}<br />
            Load Progress :
            {{ status.totalLoadedCount }}/{{ status.loadingQueueCount }}<br />
            Last Check Finished : {{ status.lastCheckDate | date: "medium"
            }}<br />
        </p>
    </div>
</div>

<ng-template #emptyTpl>
    <nz-empty></nz-empty>
</ng-template>
