import { Component, OnInit } from "@angular/core";

@Component({
    selector: "device-admin",
    templateUrl: "device.component.html",
})
export class DeviceComponent implements OnInit {
    ngOnInit() {}
}
